# This file is part of reLyX, copyright (c) 1998 Amir Karger
# You are free to use and modify this code under the terms of
# the GNU General Public Licence version 2 or later.

package LastLyX;
# This package is the last step in creating a LyX file. It:
#  - rejoins the lyx preamble to the rest of the file
#  - prints out the "header" information for tables
#  - translates any remaining unknown commands (well, not yet -Ak 980131)
#  - creates a '.lyx' file

use strict;
use RelyxTable; # handle LaTeX tables
use File::Basename;

my $debug_on; # was -d option given?

sub last_lyx {
# Arg0 is input file name
# Arg1 is output file name (foo.lyx)
# Arg2 is a string containing the entire preamble

    my ($InFileName, $OutFileName, $LyXPreamble) = (shift, shift, shift);
    $debug_on = (defined($main::opt_d) && $main::opt_d);
    my $zzz=$debug_on ? " LyX file ($InFileName --> $OutFileName)\n" :"... ";
    print STDERR "Writing$zzz";
    open (INFILE, "<$InFileName");
    open (OUTFILE, ">$OutFileName");

    # Print the preamble
    print OUTFILE $LyXPreamble;

    # Now print out the rest of the LyX file
    # Some lines have to be changed somewhat
    # Otherwise just print all lines as they appear.
    # In the future, we could buffer text, and then get rid of extra
    #     '\latex default \latex latex' or '\end_deeper \begin_deeper' pieces
    #     created by the translator
    while (<INFILE>) {
	if (/$RelyxTable::TableBeginString/o) {
	    # Write out the header information for the table
	    $_ = &print_table;

	} elsif (/$BasicLyX::bibstyle_insert_string/o) {
	    # Replace the "insert bibstyle file here" with the actual file name
	    
	    my $ins = $BasicLyX::bibstyle_insert_string;
	    my $fil = $BasicLyX::bibstyle_file;
	    if ($fil) {
		s/$ins/$fil/;
	    } else {
		warn("Don't know which bibliographystyle file to use!\n".
		 "Replace '$ins' in the LyX file with the bibstyle file\n");
	    }

	} elsif (/^\Q$BasicLyX::Begin_Inset_Include\E/o) {
	    # tell main:: we need to translate an included (or inputted) file
	    m/\{(.*)\}\s*$/ or warn "weird Include command $_";
	    my $fil = $1;
	    print "Adding $fil to file list\n" if $debug_on;
	    push @main::File_List, $fil;

	    # include file.lyx, not file.tex!
	    my ($basename, $path, $suffix) = fileparse($fil, @main::suffixlist);
	    $path .= '/' unless $path =~ /\/$/; # BUG in perl5.002 fileparse!
	    s/\Q{$fil}\E/{$path$basename.lyx}/;
	} # end special if for table, bibstyle, include

	print OUTFILE $_;
    }

    close INFILE; close OUTFILE;
    #warn "Done writing LyX file!\n";
} # end sub last_lyx

sub print_table {
# Print a table, from TableBeginString to TableEndString
# Also (kind of a hack) remove the last \newline in a table, if any,
#    since it causes LyX to seg fault.
    my $to_print; # string to collect the table in
    my $thistable = shift(@RelyxTable::table_array);

    # First, it needs the header information
    $to_print = $thistable->print_info;

    # Next, print out the whole table
    my $line;
    while (($line = <INFILE>) !~ /$RelyxTable::TableEndString/o) {
        $to_print .= $line;
    }

    # Finally, remove the last \newline, if any
    # (which is created by a \\ \hline at the end of a table)
    $to_print =~ s/\\newline(?=\s*$)//;

    return $to_print;
} # end sub print_table

1; # return true to main package
